﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using Com.Piwigo.Uploader.DTO;
using Com.Piwigo.Lib.DTO;

namespace Com.Piwigo.Uploader.Service
{

    public class LoadFromDirectory : IService 
    {
        string[] extensions = { ".jpeg", ".jpg", ".JPEG", ".JPG" };

        private LocalImage loadFile(String fileName)
        {
            LocalImage img = new LocalImage();
            // loading image data in album
            if (extensions.Contains<String>(Path.GetExtension(fileName)))
            {
                FileInfo fi = new FileInfo(fileName);
                img.LocalFile = fi.FullName;
                img.Name = fi.Name;
                img.ConfidentialLevel = ModelManager.Instance.UploadOption.ConfidentialLevel;
                img.Author = ModelManager.Instance.UploadOption.Author;
                img.AuthorComment = "";
           }
            return img;
        }

        private LocalAlbum loadDirectory(String dirName, Int32? rootAlbumId)
        {
            DirectoryInfo dir = new DirectoryInfo(dirName);
            LocalAlbum album = new LocalAlbum() { LocalDir = dir.FullName, Name = dir.Name };

            Int32 parentAlbumId =  (rootAlbumId.HasValue ? rootAlbumId.Value : PwgAlbum.RootAlbumId);

            album.MessageInfo = "";
            album.UpperCatId = parentAlbumId;
            
            // loading image data in album
            album.LocalImages = new LocalImageList();
            foreach (string strfile in Directory.EnumerateFiles(dir.FullName, "*.*", SearchOption.TopDirectoryOnly)
                .Where(s => extensions.Any(ext => ext == Path.GetExtension(s))))
            {
                album.LocalImages.Add(loadFile(strfile));
            }

            album.SubAlbums = new LocalAlbumList();
            foreach (string strdir in Directory.EnumerateDirectories(dir.FullName))
            {
                album.SubAlbums.Add(loadDirectory(strdir, null));
            }

            return album;
        }

        public void runService()
        {
            if (File.Exists(ModelManager.Instance.directoryName))
            {
                if (extensions.Contains<String>(Path.GetExtension(ModelManager.Instance.directoryName)))
                {
                    FileInfo file = new FileInfo(ModelManager.Instance.directoryName);
                    LocalAlbum album;
                    album = ModelManager.Instance.lstAlbumLocaux.SingleOrDefault(a => a.Name == file.Directory.Name);

                    if (album == null)
                    {
                        album = new LocalAlbum() { LocalDir = file.Directory.FullName, Name = file.Directory.Name };
                        album.MessageInfo = "auto created for photo input.";
                        album.UpperCatId = 0;
                        album.LocalImages = new LocalImageList();
                        album.SubAlbums = new LocalAlbumList();

                        ModelManager.Instance.lstAlbumLocaux.Add(album);
                    }

                    if (album.LocalImages.Count(a => a.Name == file.Name) < 1)
                    {
                        album.LocalImages.Add(loadFile(file.FullName));
                    }
                }
            }
            else if (Directory.Exists(ModelManager.Instance.directoryName))
            {
                Int32 RootAlbumId = PwgAlbum.RootAlbumId;
                LocalAlbum rootAlbum = null;

                if (ModelManager.Instance.SelectedNode != null)
                {
                    if (ModelManager.Instance.SelectedNode.Tag != null)
                    {
                        if (ModelManager.Instance.SelectedNode.Tag is LocalAlbum)
                        {
                            rootAlbum = (LocalAlbum)ModelManager.Instance.SelectedNode.Tag;
                            RootAlbumId = (rootAlbum.Id.HasValue ? rootAlbum.Id.Value : PwgAlbum.RootAlbumId);
                        }
                        else if (ModelManager.Instance.SelectedNode.Tag is PwgImage)
                        {
                            rootAlbum = (LocalAlbum)ModelManager.Instance.SelectedNode.Parent.Tag;
                            RootAlbumId = (rootAlbum.Id.HasValue ? rootAlbum.Id.Value : PwgAlbum.RootAlbumId);
                        }
                    }
                }

                if (rootAlbum != null)
                {
                    rootAlbum.SubAlbums.Add(
                        loadDirectory(ModelManager.Instance.directoryName,
                        RootAlbumId)
                        );
                }
                else
                {
                    ModelManager.Instance.lstAlbumLocaux.Add(
                        loadDirectory(ModelManager.Instance.directoryName,
                        PwgAlbum.RootAlbumId)
                        );
                }
            }
        }
    }
}
